/**
 * Downloads a project boilerplate repository into destination
 * @param  {Object}   opts     {repository: [String, Object], destination: String}
 * @param  {Function} done End state done
 * @return {null}
 */
module.exports = function(opts, done) {
	'use strict';

	var ghdownload = require('github-download'),
		mkdirp = require('mkdirp').sync,
		exists = require('fs').existsSync,
		fs = require('fs'),
		fse = require('fs-extra'),
		_ = require('lodash'),
		path = require('path'),
		buildScriptsRepository = 'https://github.com/siteone-feds/feds-build-scripts',
		buildDestination = path.join(opts.destination, './feds/build');

	if (!_.has(opts, 'repository')) {
		return done(new Error('Repository not defined in the opts.'));
	}

	if (!_.has(opts, 'name')) {
		return done(new Error('Project must have a name.'));
	}

	if (!_.has(opts, 'destination')) {
		opts.destination = path(process.cwd(), './frontend');
	}

	if (!exists(opts.destination)) {
		console.log('Creating destination dir: ' + opts.destination);
		mkdirp(opts.destination);
	}

	var result = null;

	ghdownload(opts.repository, opts.destination)
		.on('error', function(err) {
			result = err;
		})
		.on('end', function() {
			try {
				// read templates and convert them to lodash templates
				var template_pkg = _.template(fs.readFileSync(path.join(opts.destination, '/package.template'))),
					template_feds = _.template(fs.readFileSync(path.join(opts.destination, '/feds.template')));

				// read boilerplate info and remove the file
				try {
					opts.boilerplateVersion = fse.readJsonSync(path.join(opts.destination, '/package.json')).version;
					fse.removeSync(path.join(opts.destination, '/package.json'));
				} catch (e) {
					throw e;
				}

				// write filled project descriptors
				fs.writeFileSync(path.join(opts.destination, '/package.json'), template_pkg(opts), 'utf-8');
				fs.writeFileSync(path.join(opts.destination, '/feds.json'), template_feds(opts), 'utf-8');

				// remove templates
				fse.removeSync(path.join(opts.destination, '/package.template'));
				fse.removeSync(path.join(opts.destination, '/feds.template'));
			} catch (e) {
				// creating project descriptors failed
				done(e);
			}

			mkdirp(buildDestination);
			// ghdownload(buildScriptsRepository, buildDestination)
			// 	.on('error', function (err) {
			// 		result = err;
			// 	})
			// 	.on('end', function () {
			// 		// everything went better then expected
			// 	});
			var join = function(sep) { return function(arr) { return arr.join(sep); }; };
			var pkg = fse.readJsonSync(path.join(opts.destination, 'package.json'));
			var packagesToInstall = _(pkg.dependencies)
				.merge(pkg.devDependencies)
				.pairs()
				.map(join('@'))
				.value();

			require('spawn-npm-install').install(packagesToInstall, {
				stdio: 'inherit',
				cwd: opts.destination
			}, function() {
				done(result);
			});
		});
};
