'use strict';

var C = require('./palette');
var Case = require('case');
var path = require('path');
var fs = require('fs');
var _ = require('lodash');
var pluralize = require('pluralize');
var slugify = require('slugify');
var mkdirp = require('mkdirp');

/**
 * Generates files from templates
 * @param  {Object} 	opts Options object containing type, name, types list and sources
 * @param  {Function} 	done Complete callback
 */
module.exports = function(opts, done) {
	var names = _.map(opts.names, function(i) {
			return path.basename(i, '.' + path.extname(i));
		}),
		dests = _.map(opts.names, function(i) {
			return path.join(process.cwd(), opts.destRoot, path.dirname(i), Case.lower(path.basename(i, '.' + path.extname(i)), '-') + '/');
		}),
		source_files = _.map(opts.files, function(i) {
			return path.join(process.cwd(), i);
		}),
		extensions = _.map(opts.files, function(i) {
			return path.extname(i);
		}),
		templates = {};

	try {
		// go thru all extensions in type and cache templates
		_.each(source_files, function(source_file) {
			var template = fs.readFileSync(source_file, 'utf-8');
			templates[path.extname(source_file)] = _.template(template);
		});

		// create template for each requested names and each extension type contains
		_.each(names, function(name, i) {
			console.log(C.key('Generate resource: ') + C.value(opts.type));
			console.log(C.key('Resource name: ') + C.value(name));
			_.each(extensions, function(extension) {
				// target path
				var filePath = dests[i];
				var fileName = slugify(name, '-') + extension;
				var pagesPath = filePath.split('/');
				var dirs = [];

				while (pagesPath[pagesPath.length - 1] !== 'pages' && pagesPath.length > 0) {
					var popped = pagesPath.pop();
					!_.isEmpty(popped) && dirs.push(popped);
				}

				// template string + data for template
				var data = templates[extension]({
					partName: Case.pascal(name),
					partNameSlug: slugify(name, '-'),
					partType: opts.type,
					partTypePlural: pluralize(opts.type),
					partNameScript: dirs.join('-')
				});

				mkdirp.sync(filePath);
				// write file
				fs.writeFileSync(path.join(filePath, fileName), data, 'utf-8');
			});
		});
	}
	// report failure if something break during generation
	catch (e) {
		done(e);
		return;
	}

	done();
};
